"""Simple PEM encode, decode and extraction
"""

"""
RCSid:
	$Id: pem.py,v 1.3 2012/10/26 16:45:18 sjg Exp $

	@(#) Copyright (c) 2012 Simon J. Gerraty

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net

"""

import base64

def pem_encode(id, data):
    """PEM encode data with given id"""
    pem = ['-----BEGIN {0}-----'.format(id)]
    b64 = base64.b64encode(data)
    x = 0
    n = len(b64)
    while x < n:
        y = x + min(n - x, 64)
        pem.append(b64[x:y])
        x = y
        
    pem.append('-----END {0}-----\n'.format(id))
    return '\n'.join(pem)

def pem_extract(id, lines, debug=0):
    """return the portion of lines which contains id"""
    s = -1
    for i in range(0, len(lines)):
        if s >= 0:
            if lines[i].startswith('-----END'):
                if debug:
                    print i,lines[i]
                return lines[s:i+1]
            
        if not lines[i].startswith('-----BEGIN'):
            continue
        if lines[i].find(id) < 0:
            continue
        s = i
        if debug:
            print i,lines[i]

    return None

def pem_decode(id, lines, debug=0):
    """extract and decode pem data"""
    pem = pem_extract(id, lines, debug)
    if not pem:
        return None
    if debug:
        print pem
    data = []
    for line in pem:
        if line.startswith('-----'):
            continue
        data.append(base64.b64decode(line))
    return ''.join(data)

if __name__ == '__main__':
    m0 = """
    Hello, World!
    Life! don't talk to me about life
    Here I am, brain the size of a planet...
    and what do they have me do?"""

    print m0
    pem = pem_encode('MESSAGE', m0)
    print pem
    plines = pem.split('\n')
    m1 = pem_decode('MESSAGE', plines, True)
    print m1
    m2 = pem_decode('MESSAGE', ['junk\n'] + plines + ['more junk\n'])
    print m2
    
    assert m2 == m0

